# Standard library
from typing import (
    Optional,
)

# Third party libraries
from aiogqlc import GraphQLClient

# Local libraries
from tracers.config import (
    CONFIG,
)
from tracers.utils import (
    log,
)

REQUIRED_VARIABLES = [
    (CONFIG.api_token, CONFIG.api_token_source),
    (CONFIG.endpoint_url, CONFIG.endpoint_url_source),
    (CONFIG.system_id, CONFIG.system_id_source),
]

if all(var for var, _ in REQUIRED_VARIABLES):
    CLIENT: Optional[GraphQLClient] = GraphQLClient(
        endpoint=CONFIG.endpoint_url,
        headers={
            'authorization': f'Bearer {CONFIG.api_token}'
        },
    )
else:
    log('Please not that transactions won\'t be uploaded!!', level='warning')
    log(level='warning')
    log('The following environment variables are required:', level='warning')
    for var, source in REQUIRED_VARIABLES:
        if not var:
            log(f'- {source}', level='warning')

    CLIENT = None



# Standard library
import os
from typing import (
    NamedTuple,
    Optional,
)

# Local libraries
from tracers.constants import (
    LOGGER_DAEMON,
)


def _get(var_name: str, var_default: Optional[str] = None) -> Optional[str]:
    var_value: Optional[str] = os.environ.get(var_name, var_default)

    if var_name in [
        'TRACERS_API_TOKEN',
    ]:
        LOGGER_DAEMON.info('%s = %s', var_name, "<set>" if var_value else "")
    else:
        LOGGER_DAEMON.info('%s = %s', var_name, var_value)

    return var_value


# Runtime constants
class Config(NamedTuple):
    api_token: Optional[str]
    endpoint_url: Optional[str]
    system_id: Optional[str]

    api_token_source: str = 'TRACERS_API_TOKEN'
    endpoint_url_source: str = 'TRACERS_ENDPOINT_URL'
    system_id_source: str = 'TRACERS_SYSTEM_ID'


CONFIG = Config(
    api_token=_get('TRACERS_API_TOKEN'),
    endpoint_url=_get('TRACERS_ENDPOINT_URL'),
    system_id=_get('TRACERS_SYSTEM_ID', 'default'),
)
